@@ -1,7 +1,7 @@
 // -*- C++ -*-
 //
 // This file is part of YODA -- Yet more Objects for Data Analysis
-// Copyright (C) 2008-2024 The YODA collaboration (see AUTHORS for details)
+// Copyright (C) 2008-2025 The YODA collaboration (see AUTHORS for details)
 //
 #ifndef YODA_BINNEDAXIS_H
 #define YODA_BINNEDAXIS_H
@@ -592,7 +592,7 @@ namespace YODA {
 
   template <typename T>
   Axis<T, isCAxis<T>>::Axis(std::vector<std::pair<EdgeT, EdgeT>> binsEdges) {
-    if (binsEdges.empty()) throw BinningError("Expected at least one discrete edge");
+    if (binsEdges.empty()) throw BinningError("Expected at least one pair of edges");
 
     std::sort(binsEdges.begin(), binsEdges.end(), [](auto &left, auto &right) {
       return left.first < right.first;
@@ -662,6 +662,8 @@ namespace YODA {
   template <typename T>
   size_t Axis<T, isCAxis<T>>::index(const EdgeT& x) const {
       if (_edges.size() <= 2) throw BinningError("Axis initialised without specifying edges");
+      // Only one edge (i.e. axis has under- and overflow bin, but no visible bin)
+      if (_edges.size() == 3) return x >= _edges[1];
       // Check overflows
       if (std::isinf(x)) { return x < 0? 0 : _edges.size() - 2; }
       // Get initial estimate
@@ -790,31 +792,40 @@ namespace YODA {
 
   template <typename T>
   void Axis<T, isCAxis<T>>::_setEstimator() noexcept {
-    if (_edges.empty()) {
-        _est = std::make_shared<LinBinEstimator>(0, 0, 1);
-      } else if (_edges.front() <= 0.0) {
-        _est = std::make_shared<LinBinEstimator>(_edges.size()-1, _edges.front(), _edges.back());
-      } else {
-        LinBinEstimator linEst(_edges.size()-1, _edges.front(), _edges.back());
-        LogBinEstimator logEst(_edges.size()-1, _edges.front(), _edges.back());
 
-        // Calculate mean index estimate deviations from the correct answers (for bin edges)
-        double logsum = 0, linsum = 0;
-        for (size_t i = 0; i < _edges.size(); i++) {
-          logsum += logEst(_edges[i]) - i;
-          linsum += linEst(_edges[i]) - i;
-        }
-        const double log_avg = logsum / _edges.size();
-        const double lin_avg = linsum / _edges.size();
+    // Empty set for nullary constructor
+    if (_edges.size() <= 2) {
+      _est = std::make_shared<LinBinEstimator>(0, 0, 1);
+      return;
+    }
 
-        // This also implicitly works for NaN returned from the log There is a
-        // subtle bug here if the if statement is the other way around, as
-        // (nan < linsum) -> false always.  But (nan > linsum) -> false also.
-        if (log_avg < lin_avg) { //< Use log estimator if its avg performance is better than lin
-          _est = std::make_shared<LogBinEstimator>(logEst);
-        } else { // Else use linear estimation
-          _est = std::make_shared<LinBinEstimator>(linEst);
-        }
+    // There is at least one visible edge
+    int front = 1, back = (int)_edges.size()-2;
+    if (_edges[front] <= 0.0) {
+      _est = std::make_shared<LinBinEstimator>(back - front, _edges[front], _edges[back]);
+    }
+    else {
+      LinBinEstimator linEst(back - front, _edges[front], _edges[back]);
+      LogBinEstimator logEst(back - front, _edges[front], _edges[back]);
+
+      // Calculate mean index estimate deviations from the correct answers (for bin edges)
+      double logsum = 0, linsum = 0;
+      for (int i = front; i <= back; ++i) {
+        logsum += abs(logEst(_edges[i]) - double(i-1));
+        linsum += abs(linEst(_edges[i]) - double(i-1));
+      }
+      const double log_avg = logsum / _edges.size();
+      const double lin_avg = linsum / _edges.size();
+
+      // This also implicitly works for NaN returned from the log There is a
+      // subtle bug here if the if statement is the other way around, as
+      // (nan < linsum) -> false always.  But (nan > linsum) -> false also.
+      if (log_avg < lin_avg) { //< Use log estimator if its avg performance is better than lin
+        _est = std::make_shared<LogBinEstimator>(logEst);
+      }
+      else { // else use linear estimation
+        _est = std::make_shared<LinBinEstimator>(linEst);
+      }
     }
   }
 
