@@ -1,7 +1,7 @@
 // -*- C++ -*-
 //
 // This file is part of YODA -- Yet more Objects for Data Analysis
-// Copyright (C) 2008-2024 The YODA collaboration (see AUTHORS for details)
+// Copyright (C) 2008-2025 The YODA collaboration (see AUTHORS for details)
 //
 #ifndef YODA_BinnedDbn_h
 #define YODA_BinnedDbn_h
@@ -13,11 +13,16 @@
 #include "YODA/BinnedEstimate.h"
 #include "YODA/Scatter.h"
 
+#ifdef HAVE_HDF5
+#include "YODA/Utils/H5Utils.h"
+#endif
+
 #include <memory>
 #include <utility>
 #include <iostream>
 #include <iomanip>
 
+
 namespace YODA {
 
   /// All histograms can be instantiated through this alias.
@@ -704,6 +709,44 @@ namespace YODA {
       tmp._renderYODA(os, width);
     }
 
+    #ifdef HAVE_HDF5
+    // @brief Extract axes edges of this AO into map of @a edges
+    void _extractEdges(std::map<std::string, EdgeHandlerBasePtr>& edges,
+                       const std::vector<std::string>&) const noexcept {
+
+      using lenT = EdgeHandler<size_t>;
+      using lenPtr = EdgeHandlerPtr<size_t>;
+      const std::string lengthID("sizeinfo");
+      lenPtr nedges = std::static_pointer_cast<lenT>(edges.find(lengthID)->second);
+
+      auto extractEdges = [&edges, &binning = BaseT::_binning, &nedges](auto I) {
+
+        using thisEdgeT = typename BinningT::template getEdgeT<I>;
+        using thisHandlerT = EdgeHandler<thisEdgeT>;
+        using thisHandlerPtr = EdgeHandlerPtr<thisEdgeT>;
+
+        const std::string edgeID = std::string("edges_") + TypeID<thisEdgeT>::name();
+        std::vector<thisEdgeT> tmp = binning.template edges<I>();
+        nedges->extend({ tmp.size() });
+
+        auto itr = edges.find(edgeID);
+        if (itr != edges.cend()) {
+          thisHandlerPtr edgeset = std::static_pointer_cast<thisHandlerT>(itr->second);
+          edgeset->extend(std::move(tmp));
+        }
+        else {
+          edges[edgeID] = std::make_shared<thisHandlerT>(std::move(tmp));
+        }
+      };
+      MetaUtils::staticFor<sizeof...(AxisT)>(extractEdges);
+
+      std::vector<size_t> masks = BaseT::_binning.maskedBins();
+      nedges->extend({ masks.size() });
+      nedges->extend(std::move(masks));
+
+    };
+    #endif
+
     /// @}
 
     /// @name MPI (de-)serialisation
@@ -751,9 +794,13 @@ namespace YODA {
     /// @brief Produce a BinnedEstimate from a DbnStorage
     ///
     /// The binning remains unchanged.
-    BinnedEstimate<AxisT...> mkEstimate(const std::string& path="",
+    ///
+    /// @note The @a overflowsWidth argument will be applied
+    /// to all bins outside the visible bin range.
+    BinnedEstimate<AxisT...> mkEstimate(const std::string& path = "",
                                         const std::string& source = "",
-                       [[maybe_unused]] const bool divbyvol=true) const {
+                       [[maybe_unused]] const bool divbyvol = true,
+                                        const double overflowsWidth = -1.0) const {
 
       // @todo Should we check BaseT::nanCount() and report?
       BinnedEstimate<AxisT...> rtn(BaseT::_binning);
@@ -772,7 +819,7 @@ namespace YODA {
       }
 
       for (const auto& b : BaseT::bins(true, true)) {
-        if (!b.isVisible())  continue;
+        if (overflowsWidth <= 0. && !b.isVisible())  continue;
         if constexpr(DbnN > sizeof...(AxisT)) {
           rtn.bin(b.index()).setVal(b.mean(DbnN));
           if (b.numEntries()) { // only set uncertainty for filled Dbns
@@ -780,7 +827,7 @@ namespace YODA {
           }
         }
         else {
-          const double scale = divbyvol? b.dVol() : 1.0;
+          const double scale = divbyvol? (b.isVisible()? b.dVol() : overflowsWidth) : 1.0;
           rtn.bin(b.index()).setVal(b.sumW() / scale);
           if (b.numEntries()) { // only set uncertainty for filled Dbns
             rtn.bin(b.index()).setErr(b.errW() / scale, source);
@@ -795,21 +842,29 @@ namespace YODA {
     /// and return as a vector.
     ///
     /// The binning dimension is reduced by one unit.
+    ///
+    /// @note The @a overflowsWidth argument will be applied
+    /// to all bins outside the visible bin range.
     template<size_t axisN, typename = std::enable_if_t< (axisN < sizeof...(AxisT)) >>
-    auto mkEstimates(const std::string& path="", const std::string source = "",
-                     const bool divbyvol=true, const bool includeOverflows=false) {
+    auto mkEstimates(const std::string& path = "", const std::string source = "",
+                     const bool divbyvol = true, const bool includeOverflows = false,
+                     const double overflowsWidth = -1.0) {
 
-      BinnedEstimate<AxisT...> est = mkEstimate(path, source, divbyvol);
+      BinnedEstimate<AxisT...> est = mkEstimate(path, source, divbyvol, overflowsWidth);
       return est.template mkEstimates<axisN>(path, includeOverflows);
     }
 
 
     /// @brief Produce a ScatterND from a DbnStorage
-    auto mkScatter(const std::string& path="", const bool divbyvol=true,
-                                               const bool usefocus=false,
-                                               const bool includeOverflows=false,
-                                               const bool includeMaskedBins=false) const {
-      const BinnedEstimate<AxisT...> est = mkEstimate("", "", divbyvol);
+    ///
+    /// @note The @a overflowsWidth argument will be applied
+    /// to all bins outside the visible bin range.
+    auto mkScatter(const std::string& path="", const bool divbyvol = true,
+                                               const bool usefocus = false,
+                                               const bool includeOverflows = false,
+                                               const bool includeMaskedBins = false,
+                                               const double overflowsWidth = -1.0) const {
+      const BinnedEstimate<AxisT...> est = mkEstimate("", "", divbyvol, overflowsWidth);
       ScatterND<sizeof...(AxisT)+1> rtn = est.mkScatter(path, "", includeOverflows, includeMaskedBins);
       if (usefocus) {
         size_t idx = 0;
@@ -1022,6 +1077,33 @@ namespace YODA {
     }
 
 
+    /// @brief Convert the BinnedDbn to a BinnedEstimate representing
+    /// the effective number of entries in each bin
+    ///
+    /// @note The @a overflowsWidth argument will be applied
+    /// to all bins outside the visible bin range.
+    BinnedEstimate<AxisT...> mkBinnedEffNumEntries(const std::string& path="",
+                                                   const std::string& source = "",
+                                                   const bool includeOverflows = true,
+                                                   const bool divbyvol = true,
+                                                   const double overflowsWidth = -1.0) {
+
+      BinnedEstimate<AxisT...> rtn = mkEstimate(path);
+
+      for (const auto& b : BaseT::bins(includeOverflows)) {
+        double scale = 1.0;
+        if (divbyvol) {
+          scale = (overflowsWidth > 0. && !b.isVisible())? overflowsWidth : b.dVol();
+        }
+        const double effN = b.effNumEntries() / scale;
+        const double err = effN * b.relErrW() / scale;
+        rtn.bin(b.index()).set(effN, {-err, err}, source);
+      }
+
+      return rtn;
+    }
+
+
     /// @brief Return an inert version of the analysis object (e.g. scatter, estimate)
     AnalysisObject* mkInert(const std::string& path = "",
                             const std::string& source = "") const noexcept {
@@ -1103,21 +1185,21 @@ namespace YODA {
       const size_t idx = b_num.index();
       const auto& b_den = denom.bin(idx);
       double v, e;
-      if (!b_den.effNumEntries()) {
+      if (isZero(b_den.effNumEntries())) {
         v = std::numeric_limits<double>::quiet_NaN();
         e = std::numeric_limits<double>::quiet_NaN();
       }
       else {
         if constexpr(DbnN > sizeof...(AxisT)) {
           v = b_num.mean(DbnN) / b_den.mean(DbnN);
-          const double e_num = b_num.effNumEntries()? b_num.relStdErr(DbnN) : 0;
-          const double e_den = b_den.effNumEntries()? b_den.relStdErr(DbnN) : 0;
+          const double e_num = isZero(b_num.effNumEntries())? 0 : b_num.relStdErr(DbnN);
+          const double e_den = isZero(b_den.effNumEntries())? 0 : b_den.relStdErr(DbnN);
           e = fabs(v) * sqrt(sqr(e_num) + sqr(e_den));
         }
         else {
           v = b_num.sumW() / b_den.sumW();
-          const double e_num = b_num.sumW()? b_num.relErrW() : 0;
-          const double e_den = b_den.sumW()? b_den.relErrW() : 0;
+          const double e_num = isZero(b_num.effNumEntries())? 0 : b_num.relErrW();
+          const double e_den = isZero(b_den.effNumEntries())? 0 : b_den.relErrW();
           e = fabs(v) * sqrt(sqr(e_num) + sqr(e_den));
         }
       }
@@ -1180,15 +1262,10 @@ namespace YODA {
       // If no entries on the denominator, set eff = err = 0 and move to the next bin
       double eff = std::numeric_limits<double>::quiet_NaN();
       double err = std::numeric_limits<double>::quiet_NaN();
-      try {
-        if (b_tot.sumW()) {
-          eff = b_rtn.val();
-          err = sqrt(abs( ((1-2*eff)*b_acc.sumW2() + sqr(eff)*b_tot.sumW2()) / sqr(b_tot.sumW()) ));
-        }
-      } catch (const LowStatsError& e) {
-        //
+      if (!isZero(b_tot.effNumEntries())) {
+        eff = b_rtn.val();
+        err = sqrt(fabs( add((1.0-2.0*eff)*b_acc.sumW2(), sqr(eff)*b_tot.sumW2()) / sqr(b_tot.sumW()) ));
       }
-
       b_rtn.setErr({-err, err}); // @todo put "stats" as source?
     }
     return rtn;
@@ -1203,7 +1280,7 @@ namespace YODA {
   }
 
 
-  /// @brief Convert a Histo1D to a Scatter2D representing the integral of the histogram
+  /// @brief Convert a BinnedDbn to a BinnedEstimate representing the integral of the histogram
   ///
   /// @note The integral histo errors are calculated as sqrt(binvalue), as if they
   /// are uncorrelated. This is not in general true for integral histograms, so if you
@@ -1229,7 +1306,7 @@ namespace YODA {
   }
 
 
-  /// @brief Convert a Histo1D to a Scatter2D where each bin is a fraction of the total
+  /// @brief Convert a BinnedDbn to a BinnedEstimate where each bin is a fraction of the total
   ///
   /// @note This sounds weird: let's explain a bit more! Sometimes we want to
   /// take a histo h, make an integral histogram H from it, and then divide H by
