@@ -1,7 +1,7 @@
 // -*- C++ -*-
 //
 // This file is part of YODA -- Yet more Objects for Data Analysis
-// Copyright (C) 2008-2024 The YODA collaboration (see AUTHORS for details)
+// Copyright (C) 2008-2025 The YODA collaboration (see AUTHORS for details)
 //
 #ifndef YODA_BinnedEstimate_h
 #define YODA_BinnedEstimate_h
@@ -14,6 +14,11 @@
 #include "YODA/Utils/BinnedUtils.h"
 #include <regex>
 
+#ifdef HAVE_HDF5
+#include "YODA/Utils/H5Utils.h"
+#endif
+
+
 namespace YODA {
 
   /// @note Anonymous namespace to limit visibility to this file
@@ -75,14 +80,11 @@ namespace YODA {
   class EstimateStorage
       : public BinnedStorage<Estimate, AxisT...>,
         public AnalysisObject {
-  protected:
+  public:
 
     using BaseT = BinnedStorage<Estimate, AxisT...>;
     using BinningT = typename BaseT::BinningT;
     using BinT = typename BaseT::BinT;
-
-  public:
-
     using BinType = BinT;
     using AnalysisObject::operator =;
 
@@ -239,6 +241,72 @@ namespace YODA {
       tmp._renderYODA(os, width);
     }
 
+    #ifdef HAVE_HDF5
+
+    // @brief Extract error labels of this AO
+    void _extractLabels(std::vector<std::string>& labels,
+                        std::vector<size_t>& labelsizes) const noexcept {
+      size_t nlabels = 0;
+      for (const auto& b : BaseT::bins(true, true)) {
+        std::vector<std::string> keys = b.sources();
+        nlabels += keys.size()+1; //< +1 for length info itself
+        labels.insert(std::end(labels),
+                      std::make_move_iterator(std::begin(keys)),
+                      std::make_move_iterator(std::end(keys)));
+      }
+      std::sort(labels.begin(), labels.end());
+      labels.erase( std::unique(labels.begin(), labels.end()), labels.end() );
+      labelsizes.emplace_back(std::move(nlabels));
+    };
+
+    // @brief Extract axes edges of this AO into map of @a edges
+    void _extractEdges(std::map<std::string, EdgeHandlerBasePtr>& edges,
+                       const std::vector<std::string>& labels) const noexcept {
+
+      using lenT = EdgeHandler<size_t>;
+      using lenPtr = EdgeHandlerPtr<size_t>;
+      const std::string lengthID("sizeinfo");
+      lenPtr nedges = std::static_pointer_cast<lenT>(edges.find(lengthID)->second);
+
+      auto extractEdges = [&edges, &binning = BaseT::_binning, &nedges](auto I) {
+
+        using thisEdgeT = typename BinningT::template getEdgeT<I>;
+        using thisHandlerT = EdgeHandler<thisEdgeT>;
+        using thisHandlerPtr = EdgeHandlerPtr<thisEdgeT>;
+
+        const std::string edgeID = std::string("edges_") + TypeID<thisEdgeT>::name();
+        std::vector<thisEdgeT> tmp = binning.template edges<I>();
+        nedges->extend({ tmp.size() });
+
+        auto itr = edges.find(edgeID);
+        if (itr != edges.cend()) {
+          thisHandlerPtr edgeset = std::static_pointer_cast<thisHandlerT>(itr->second);
+          edgeset->extend(std::move(tmp));
+        }
+        else {
+          edges[edgeID] = std::make_shared<thisHandlerT>(std::move(tmp));
+        }
+      };
+      MetaUtils::staticFor<sizeof...(AxisT)>(extractEdges);
+
+      std::vector<size_t> masks = BaseT::_binning.maskedBins();
+      nedges->extend({ masks.size() });
+      nedges->extend(std::move(masks));
+
+      const auto& itr = labels.cbegin();
+      const auto& itrEnd = labels.cend();
+      for (const auto& b : BaseT::bins(true, true)) {
+        vector<string> keys = b.sources();
+        nedges->extend({ keys.size() });
+        for (const string& k : keys) {
+          size_t dist = std::find(itr, itrEnd, k) - itr;
+          nedges->extend({ dist });
+        }
+      }
+    }
+
+    #endif
+
     /// @}
 
 
