@@ -1,7 +1,7 @@
 // -*- C++ -*-
 //
 // This file is part of YODA -- Yet more Objects for Data Analysis
-// Copyright (C) 2008-2024 The YODA collaboration (see AUTHORS for details)
+// Copyright (C) 2008-2025 The YODA collaboration (see AUTHORS for details)
 //
 #ifndef YODA_Estimate_h
 #define YODA_Estimate_h
@@ -512,8 +512,8 @@ namespace YODA {
 
     std::vector<double> _serializeContent(bool fixed_length = false) const noexcept {
       std::vector<double> rtn;
-      const size_t nErrs = fixed_length? 1 : numErrs();
-      rtn.reserve(2*(nErrs + 1));
+      const size_t len = fixed_length? 4 : _lengthContent();
+      rtn.reserve(len);
       rtn.push_back(_value);
       if (fixed_length) {
         rtn.push_back(1.0);
@@ -567,9 +567,7 @@ namespace YODA {
         throw UserError("Expected " + std::to_string(nErrs) + " error source labels!");
 
       for (size_t i = 0; i < data.size(); ++i) {
-        const std::string name("source" + std::to_string(i+1));
-        if (!_error.count(name))
-          throw UserError("Key names have already been updated!");
+        const std::string name(nErrs==1? "" : "source" + std::to_string(i+1));
         renameSource(name, data[i]);
       }
 
@@ -601,7 +599,7 @@ namespace YODA {
         return {up, dn};
       }
       // else: both positive
-      const double env = std::max(dn, up);
+      const double env = fabs(std::max(dn, up));
       return {0., env};
     }
 
@@ -800,7 +798,7 @@ namespace YODA {
       else if (std::regex_search(src, match, re)) {
         const double acc_err = accepted.relTotalErrAvg();
         const double tot_err = total.totalErrAvg();
-        err = sqrt(std::abs( ((1-2*eff)*sqr(acc_err) + sqr(eff)*sqr(tot_err)) / sqr(tot_val) ));
+        err = sqrt(std::abs( add((1.0-2.0*eff)*sqr(acc_err), sqr(eff)*sqr(tot_err)) / sqr(tot_val) ));
         rtn.setErr({-err,err}, src);
         continue;
       }
