@@ -1,7 +1,7 @@
 // -*- C++ -*-
 //
 // This file is part of YODA -- Yet more Objects for Data Analysis
-// Copyright (C) 2008-2024 The YODA collaboration (see AUTHORS for details)
+// Copyright (C) 2008-2025 The YODA collaboration (see AUTHORS for details)
 //
 #ifndef YODA_Scatter_h
 #define YODA_Scatter_h
@@ -10,7 +10,6 @@
 #include "YODA/Point.h"
 #include "YODA/Transformation.h"
 #include "YODA/Utils/Traits.h"
-#include "YODA/Utils/sortedvector.h"
 #include "YODA/Utils/ndarray.h"
 #include <vector>
 #include <set>
@@ -191,7 +190,7 @@ namespace YODA {
     using NdVal = Utils::ndarray<double, N>;
     using NdValPair = Utils::ndarray<std::pair<double,double>, N>;
     using Point = PointND<N>;
-    using Points = Utils::sortedvector<Point>;
+    using Points = std::vector<Point>;
     using Ptr = std::shared_ptr<ScatterND>;
     using AnalysisObject::operator =;
 
@@ -377,13 +376,13 @@ namespace YODA {
 
     /// Insert a new point
     ScatterND<N>& addPoint(const PointND<N>& pt) {
-      _points.insert(pt);
+      _points.push_back(pt);
       return *this;
     }
 
     /// Insert a new rvalue point
     ScatterND<N>& addPoint(PointND<N>&& pt) {
-      _points.insert(std::move(pt));
+      _points.push_back(std::move(pt));
       return *this;
     }
 
@@ -391,7 +390,7 @@ namespace YODA {
     template <typename ValRange = ValList,
               typename = std::enable_if_t<isIterable<ValRange>>>
     ScatterND<N>& addPoint(ValRange&& pos) {
-      _points.insert(PointND<N>(std::forward<ValRange>(pos)));
+      _points.push_back(PointND<N>(std::forward<ValRange>(pos)));
       return *this;
     }
 
@@ -399,7 +398,7 @@ namespace YODA {
     template <typename ValRange = ValList,
               typename = std::enable_if_t<isIterable<ValRange>>>
     ScatterND<N>& addPoint(ValRange&& pos, ValRange&& err) {
-      _points.insert(PointND<N>(std::forward<ValRange>(pos), std::forward<ValRange>(err)));
+      _points.push_back(PointND<N>(std::forward<ValRange>(pos), std::forward<ValRange>(err)));
       return *this;
     }
 
@@ -407,9 +406,9 @@ namespace YODA {
     template <typename ValRange = ValList,
               typename = std::enable_if_t<isIterable<ValRange>>>
     ScatterND<N>& addPoint(ValRange&& pos, ValRange&& errdn, ValRange&& errup) {
-      _points.insert(PointND<N>(std::forward<ValRange>(pos),
-                                std::forward<ValRange>(errdn),
-                                std::forward<ValRange>(errup)));
+      _points.push_back(PointND<N>(std::forward<ValRange>(pos),
+                                   std::forward<ValRange>(errdn),
+                                   std::forward<ValRange>(errup)));
       return *this;
     }
 
@@ -417,7 +416,7 @@ namespace YODA {
     template <typename ValRange = ValList, typename PairRange = PairList>
     auto addPoint(ValRange&& pos, PairRange&& err)
     -> std::enable_if_t<(isIterable<ValRange,PairRange> && containsPair<PairRange>::value), ScatterND<N>>& {
-      _points.insert(PointND<N>(pos, err));
+      _points.push_back(PointND<N>(pos, err));
       return *this;
     }
 
@@ -469,15 +468,15 @@ namespace YODA {
     auto addPoint_aux(std::tuple<Args...>&& t, std::index_sequence<Is...>)
     -> std::enable_if_t<(isAllVals<Args...>::value), ScatterND<N>>& {
       if constexpr(sizeof...(Args) == 2*N) { // Case 1: symmetric errors
-        _points.insert(
+        _points.push_back(
           PointND<N>( ValVec{  static_cast<double>(std::get<Is>(t))...},
                       PairVec{{static_cast<double>(std::get<N+Is>(t)),
                                static_cast<double>(std::get<N+Is>(t))}...} ));
       }
       else { // Case 2: asymmetric errors
-        _points.insert(PointND<N>( ValVec{  static_cast<double>(std::get<Is>(t))...},
-                                   PairVec{{static_cast<double>(std::get<N+2*Is>(t)),
-                                            static_cast<double>(std::get<N+2*Is+1>(t))}...} ));
+        _points.push_back(PointND<N>( ValVec{  static_cast<double>(std::get<Is>(t))...},
+                                      PairVec{{static_cast<double>(std::get<N+2*Is>(t)),
+                                               static_cast<double>(std::get<N+2*Is+1>(t))}...} ));
       }
       return *this;
     }
@@ -499,8 +498,8 @@ namespace YODA {
     template<typename... Args, size_t... Is>
     auto addPoint_aux(std::tuple<Args...>&& t, std::index_sequence<Is...>) // Case 3: error pairs
     -> std::enable_if_t<(sizeof...(Args) == 2*N && isHalfValsHalfPairs<Args...>::value), ScatterND<N>>& {
-      _points.insert(PointND<N>( ValVec{  static_cast<double>(std::get<Is>(t))...},
-                                 PairVec{ static_cast<Pair>(std::get<N+Is>(t))...} ));
+      _points.push_back(PointND<N>( ValVec{  static_cast<double>(std::get<Is>(t))...},
+                                    PairVec{ static_cast<Pair>(std::get<N+Is>(t))...} ));
       return *this;
     }
 
@@ -532,7 +531,7 @@ namespace YODA {
       if (fixed_length)  return { }; // cannot guarantee fixed length
 
       std::vector<double> rtn;
-      rtn.reserve(numPoints() * Point::DataSize::value);
+      rtn.reserve(lengthContent());
       for (size_t i = 0; i < numPoints(); ++i) {
         std::vector<double> pdata = point(i)._serializeContent();
         rtn.insert(std::end(rtn),
