@@ -1,7 +1,7 @@
 // -*- C++ -*-
 //
 // This file is part of YODA -- Yet more Objects for Data Analysis
-// Copyright (C) 2008-2024 The YODA collaboration (see AUTHORS for details)
+// Copyright (C) 2008-2025 The YODA collaboration (see AUTHORS for details)
 //
 #ifndef YODA_MathUtils_H
 #define YODA_MathUtils_H
@@ -60,7 +60,7 @@ namespace YODA {
   /// by the absolute @a tolerance parameter, for floating point safety.
   template <typename NUM>
   inline typename std::enable_if_t<std::is_floating_point_v<NUM>, bool>
-  isZero(NUM val, double tolerance=1e-8) {
+  isZero(NUM val, double tolerance=1e-5) {
     return fabs(val) < tolerance;
   }
 
@@ -218,15 +218,10 @@ namespace YODA {
   }
 
   /// Named number-type addition in quadrature operation.
-  template <typename Num>
-  inline Num add_quad(Num a, Num b) {
-    return sqrt(a*a + b*b);
-  }
-
-  /// Named number-type addition in quadrature operation.
-  template <typename Num>
-  inline Num add_quad(Num a, Num b, Num c) {
-    return sqrt(a*a + b*b + c*c);
+  template <typename... Num>
+  inline std::enable_if_t<std::conjunction_v<std::is_arithmetic<Num>...>, double>
+  add_quad(Num ... vals) {
+    return sqrt((0.0 + ... + sqr(vals)));
   }
 
   /// Find the sign of a number
@@ -248,6 +243,17 @@ namespace YODA {
     return (val > 0) ? PLUS : MINUS;
   }
 
+  /// Subtract two numbers with FP fuzziness
+  inline double subtract(double a, double b, double tolerance = 1e-5) {
+    if (fuzzyEquals(a,b,tolerance))  return 0.;
+    return a - b;
+  }
+
+  /// Add two numbers with FP fuzziness
+  inline double add(double a, double b, double tolerance = 1e-5) {
+    return subtract(a,-b,tolerance);
+  }
+
   //@}
 
 
@@ -420,8 +426,8 @@ namespace YODA {
   /// see http://en.wikipedia.org/wiki/Weighted_mean
   inline double variance(const double sumWX, const double sumW,
                          const double sumWX2, const double sumW2) {
-    const double num = sumWX2*sumW - sqr(sumWX);
-    const double den = sqr(sumW) - sumW2;
+    const double num = subtract(sumWX2*sumW, sqr(sumWX));
+    const double den = subtract(sqr(sumW), sumW2);
     /// @todo Isn't this sensitive to the overall scale of the weights?
     /// Shouldn't it check if den is bigger then num by a set number of
     /// orders of magnitude and vice versa?
