@@ -1,7 +1,7 @@
 // -*- C++ -*-
 //
 // This file is part of YODA -- Yet more Objects for Data Analysis
-// Copyright (C) 2008-2024 The YODA collaboration (see AUTHORS for details)
+// Copyright (C) 2008-2025 The YODA collaboration (see AUTHORS for details)
 //
 #ifndef YODA_READERUTILS_H
 #define YODA_READERUTILS_H
@@ -14,6 +14,17 @@
 #include "YODA/Profile.h"
 #include "YODA/BinnedEstimate.h"
 
+#ifdef HAVE_HDF5
+#include <YODA/Config/BuildConfig.h>
+#include "YODA/Utils/H5Utils.h"
+#ifdef WITH_HIGHFIVE
+#include <YODA/highfive/H5File.hpp>
+#else
+#include <highfive/H5File.hpp>
+#define YODA_H5 HighFive
+#endif
+#endif
+
 #include <iostream>
 #include <locale>
 #include <cstring>
@@ -24,6 +35,7 @@
 #include <xlocale.h>
 #endif
 
+using std::map;
 using std::string;
 using std::vector;
 
@@ -130,6 +142,16 @@ namespace YODA {
 
     virtual AnalysisObject* assemble(const string& path = "") = 0;
 
+    #ifdef HAVE_HDF5
+
+    virtual AnalysisObject* mkFromH5(H5FileManager&) = 0;
+
+    virtual void skip(H5FileManager& h5file) {
+      h5file.skipCommon();
+    }
+
+    #endif
+
     template<typename T>
     void extractVector(const std::string& line, std::vector<T>& vec) {
       if constexpr (std::is_same<T, std::string>::value) {
@@ -188,6 +210,19 @@ namespace YODA {
       dbn = Dbn0D();
       return ao;
     }
+
+    #ifdef HAVE_HDF5
+
+    AnalysisObject* mkFromH5(H5FileManager& h5file) {
+      vector<string> annos = h5file.loadAnnotations();
+      auto* ao = new Counter(h5file.path(), annos.back());
+      annos.pop_back();
+      ao->deserializeMeta(annos);
+      ao->deserializeContent(h5file.loadContent());
+      return ao;
+    };
+
+    #endif
   };
 
 
@@ -231,6 +266,23 @@ namespace YODA {
       return ao;
     }
 
+    #ifdef HAVE_HDF5
+    AnalysisObject* mkFromH5(H5FileManager& h5file) {
+      vector<string> annos = h5file.loadAnnotations();
+      auto* ao = new Estimate0D(h5file.path(), annos.back());
+      annos.pop_back();
+      ao->deserializeMeta(annos);
+      ao->deserializeContent(h5file.loadContent());
+      ao->deserializeSources(h5file.loadSources());
+      return ao;
+    }
+
+    void skip(H5FileManager& h5file) {
+      h5file.skipCommon();
+      h5file.skipSources();
+    }
+
+    #endif
   };
 
 
@@ -267,6 +319,19 @@ namespace YODA {
       points.clear();
       return ao;
     }
+
+    #ifdef HAVE_HDF5
+
+    AnalysisObject* mkFromH5(H5FileManager& h5file) {
+      vector<string> annos = h5file.loadAnnotations();
+      auto* ao = new ScatterND<N>(h5file.path(), annos.back());
+      annos.pop_back();
+      ao->deserializeMeta(annos);
+      ao->deserializeContent(h5file.loadContent());
+      return ao;
+    }
+
+    #endif
   };
 
 
@@ -323,6 +388,29 @@ namespace YODA {
       }
     }
 
+    #ifdef HAVE_HDF5
+
+    template <size_t I>
+    void loadEdges(H5FileManager& h5file) {
+      if constexpr(I < sizeof...(AxisT)) {
+        using EdgeT = std::tuple_element_t<I, std::tuple<AxisT...>>;
+        auto& curr_edges = std::get<I>(edges);
+        curr_edges = h5file.loadEdges<EdgeT>();
+        loadEdges<I+1>(h5file);
+      }
+    }
+
+    template <size_t I>
+    void skipEdges(H5FileManager& h5file) {
+      if constexpr(I < sizeof...(AxisT)) {
+        using EdgeT = std::tuple_element_t<I, std::tuple<AxisT...>>;
+        h5file.skipEdges<EdgeT>();
+        skipEdges<I+1>(h5file);
+      }
+    }
+
+    #endif
+
     template<size_t I>
     void readDbn(std::array<double,DbnN+1>& sumW, std::array<double,DbnN+1>& sumW2) {
       if constexpr(I <= DbnN) {
@@ -441,6 +529,31 @@ namespace YODA {
       axisCheck = 0;
       return ao;
     }
+
+    #ifdef HAVE_HDF5
+
+    AnalysisObject* mkFromH5(H5FileManager& h5file) {
+      loadEdges<0>(h5file);
+      maskedBins = h5file.loadMasks();
+      vector<string> annos = h5file.loadAnnotations();
+      auto args = std::tuple_cat(edges, std::make_tuple(h5file.path(), annos.back()));
+      BaseT* ao = make_from_tuple(std::move(args));
+      annos.pop_back();
+      ao->deserializeMeta(annos);
+      ao->deserializeContent(h5file.loadContent());
+      maskedBins.clear();
+      clearEdges<0>();
+      return ao;
+    }
+
+    void skip(H5FileManager& h5file) {
+      skipEdges<0>(h5file);
+      h5file.skipMasks();
+      h5file.skipCommon();
+    }
+
+    #endif
+
   };
 
 
@@ -478,6 +591,29 @@ namespace YODA {
       }
     }
 
+    #ifdef HAVE_HDF5
+
+    template <size_t I>
+    void loadEdges(H5FileManager& h5file) {
+      if constexpr(I < sizeof...(AxisT)) {
+        using EdgeT = std::tuple_element_t<I, std::tuple<AxisT...>>;
+        auto& curr_edges = std::get<I>(edges);
+        curr_edges = h5file.loadEdges<EdgeT>();
+        loadEdges<I+1>(h5file);
+      }
+    }
+
+    template <size_t I>
+    void skipEdges(H5FileManager& h5file) {
+      if constexpr(I < sizeof...(AxisT)) {
+        using EdgeT = std::tuple_element_t<I, std::tuple<AxisT...>>;
+        h5file.skipEdges<EdgeT>();
+        skipEdges<I+1>(h5file);
+      }
+    }
+
+    #endif
+
     template <class tupleT, size_t... Is>
     BaseT* make_from_tuple(tupleT&& tuple, std::index_sequence<Is...> ) {
       BaseT* rtn = new BaseT{std::get<Is>(std::forward<tupleT>(tuple))...};
@@ -541,6 +677,38 @@ namespace YODA {
       axisCheck = 0;
       return ao;
     }
+
+    #ifdef HAVE_HDF5
+
+    AnalysisObject* mkFromH5(H5FileManager& h5file) {
+      loadEdges<0>(h5file);
+      maskedBins = h5file.loadMasks();
+
+      vector<string> annos = h5file.loadAnnotations();
+      auto args = std::tuple_cat(edges, std::make_tuple(h5file.path(), annos.back()));
+      BaseT* ao = make_from_tuple(std::move(args));
+      annos.pop_back();
+      ao->deserializeMeta(annos);
+      ao->deserializeContent(h5file.loadContent());
+
+      for (auto& b : ao->bins(true, true)) {
+        b.deserializeSources(h5file.loadSources());
+      }
+
+      maskedBins.clear();
+      clearEdges<0>();
+      return ao;
+    }
+
+    void skip(H5FileManager& h5file) {
+      skipEdges<0>(h5file);
+      h5file.skipMasks();
+      h5file.skipSources();
+      h5file.skipCommon();
+    }
+
+    #endif
+
   };
 
 
