@@ -45,7 +45,7 @@ def mkCurves2D(aos_dict, plot_ref=True, error_bars=True, colors=[],
     ref_zvals = aos[0].zVals()
     ref_zerrs = aos[0].zErrs()
     data.add(f"import numpy as np",
-             f"from numpy import nan",
+             f"from numpy import nan, inf",
              f"xpoints = {[val for val in xpoints]}",
              f"xedges  = {[val for val in xedges]}",
              f"xmins   = {[val for val in aos[0].xMins()]}",
@@ -112,7 +112,8 @@ def mkCurves2D(aos_dict, plot_ref=True, error_bars=True, colors=[],
                 f"ybin_cent = np.unique(dataf['ypoints'])",
                 f"X, Y = np.meshgrid(xbin_cent, ybin_cent)",
                 f"Z = np.array(dataf['zpoints']).reshape(X.shape[::-1])",
-                f"pc = ax.pcolormesh(X, Y, Z.T, cmap=cmap, shading='auto'"+suff,
+                f"pc = ax.pcolormesh(X, Y, Z.T, cmap=cmap, shading='auto',",
+                f"                   vmin=zLims[0], vmax=zLims[1]"+suff,
                 f"cbar = fig.colorbar(pc, orientation='vertical', ax=ax, pad=0.01)",#+suff,
                 f"cbar.set_label(ax_zLabel)")
 
@@ -153,9 +154,9 @@ def mkCurves2D(aos_dict, plot_ref=True, error_bars=True, colors=[],
         data.add(f"  '{label}' : {[val for val in zdowns]},")
     data.add("}")
 
-    cmd.add("# style options for curves",
-            "# starts at zorder>=5 to draw curve on top of legend")
     if c_styles:
+        cmd.add("# style options for curves",
+                "# starts at zorder>=5 to draw curve on top of legend")
         cmd.add("styles = {")
         for i, (key, val) in enumerate(c_styles.items()):
             cmd.add(f"  '{key}': " + "{")
@@ -356,7 +357,7 @@ def mkCurves1D(aos, ratio_panels, error_bars=True, variation_dict=None,
 
     # write dictionary of labels + yvalues to iterate over in executable python script
     data.add("import numpy as np",
-             "from numpy import nan")
+             "from numpy import nan, inf")
     data.newline()
     data.add("add_legend_handle = [",
              ",\n".join([ f"    '{k}'" for k,v in aos.items() if v.annotation('Title', None) ]),
