@@ -1,7 +1,7 @@
 import os, stat, shutil
 
 import yoda
-from yoda.plotting.mlp_preprocessor import preprocess
+from yoda.plotting.mlp_preprocessor import preprocess, applyRegexes
 import yoda.plotting.fetch_data as fd
 import yoda.plotting.utils as putils
 
@@ -12,6 +12,9 @@ mpl.use('Agg')
 import math, re
 import numpy as np
 
+RE_PATTERN_PATH = re.compile(r'PATH\[(.+?)\]')
+RE_PATTERN_NAME = re.compile(r'NAME\[(.+?)\]')
+
 
 def setCanvas(cmd, fig_dims, margins, nRatios = 0, fig_specs = None):
     """Return Pythonic string corresponding to
@@ -84,6 +87,18 @@ def mkPlottingScript1D(hist_data, hist_features, yaml_dicts, outdir, plot_name,
     cmd = yoda.util.StringCommand()
     setStyleFile(cmd, yaml_dicts, outdir)
     plot_features = yaml_dicts.get('plot features', {})
+
+    # check if user supplied regex-based label customisation
+    re_captures = []
+    for item in plot_features.get('PlotRegexes', []):
+        for match in RE_PATTERN_NAME.findall(item):
+            sub = re.findall(match, list(hist_data.values())[0].name())
+            if sub:  re_captures.append(sub[0])
+        for match in RE_PATTERN_PATH.findall(item):
+            sub = re.findall(match, list(hist_data.values())[0].path())
+            if sub:  re_captures.append(sub[0])
+    _apply_regexes(plot_features, re_captures, plot_features.get('PlotRegexSubs', {}))
+
     # adjust font sizes if necessary:
     for key, val in {
         'AxisLabelSize'  : 'axes.labelsize',  # fontsize of the axis labels
@@ -98,7 +113,6 @@ def mkPlottingScript1D(hist_data, hist_features, yaml_dicts, outdir, plot_name,
     if not has_latex:
         cmd.newline()
         cmd.add(f"plt.rcParams['text.usetex'] = False",
-                f"plt.style.use(['classic'])",
                 f"plt.rc('font', family='DejaVu Sans')")
 
     cmd.add("# plot metadata",
@@ -107,18 +121,52 @@ def mkPlottingScript1D(hist_data, hist_features, yaml_dicts, outdir, plot_name,
             f"""ax_yLabel = r'{plot_features.get("YLabel", "")}'""",
             f"""ax_title  = r'{plot_features.get("Title", "")}'""")
 
+    ax_format = {}  # Stores the items for formatting the axes in a dict
+    ax_format['logx'] = int(plot_features.get('LogX', 0))
+    ax_format['logy'] = int(plot_features.get('LogY', 1))
+
     hasNonZero = any([np.any(h.yVals()) for h in hist_data.values()])
-    xscale = 'log' if int(plot_features.get('LogX', 0)) else 'linear'
-    yscale = 'log' if int(plot_features.get('LogY', 1)) and hasNonZero else 'linear'
+    xscale = 'log' if ax_format['logx'] else 'linear'
+    yscale = 'log' if ax_format['logy'] and hasNonZero else 'linear'
     cmd.add(f"""ax_xScale = '{xscale}'""",
             f"""ax_yScale = '{yscale}'""")
 
-    ax_format = {}  # Stores the items for formatting the axes in a dict
+    ## Work out xMin and xMax from all curves, excluding under/overflow infinities
+    nOverflows = [ h.annotation('nOverflows', 0) for h in hist_data.values() ]
+    # keeping overflows requires user to have set IncludeOverflows=1 and all objects having two overflows
+    hasOverflows = set(nOverflows).pop() == 2 if int(plot_features.get('IncludeOverflows', '0')) else False
+    xmin = plot_features.get('XMin', min([min(h.xMins()[1:-1] \
+                                          if nOverflows[i] == 2 and int(plot_features.get('IncludeOverflows', '0')) else \
+                                          h.xMins()) for i,h in enumerate(hist_data.values())]))
+    xmax = plot_features.get('XMax', max([max(h.xMaxs()[1:-1] \
+                                          if nOverflows[i] == 2 and int(plot_features.get('IncludeOverflows', '0')) else \
+                                          h.xMaxs()) for i,h in enumerate(hist_data.values())]))
+    if ax_format['logx']:
+        xmin = np.log10(xmin)
+        xmax = np.log10(xmax)
+
+    # work out overflow bin width as fraction of overall binned range
+    overflowWidth = float(plot_features.get('OverflowWidthFrac', 0.07)) * (xmax - xmin) if hasOverflows else 0.0
+    underflowWidth = float(plot_features.get('UnderflowWidthFrac', 0.07)) * (xmax - xmin) if hasOverflows else 0.0
+
+    # adjust limits
+    xmin -= underflowWidth
+    xmax += overflowWidth
 
     ## Set plot limits
-    xmin = plot_features.get('XMin', min([min(h.xMins()) for h in hist_data.values()]))
-    xmax = plot_features.get('XMax', max([max(h.xMaxs()) for h in hist_data.values()]))
-    ax_format['xlim'] = (float(xmin), float(xmax))
+    ax_format['xlim'] = (float(10 ** xmin if ax_format['logx'] else xmin),
+                         float(10 ** xmax if ax_format['logx'] else xmax))
+    if any(nOverflows):
+        # set dummy xMin/xMax in case overflows are to be plotted
+        putils.adjustOverflows(hist_data.values(), ax_format['xlim'],
+                               int(plot_features.get('IncludeOverflows', '0')),
+                               hasOverflows and underflowWidth > 0,
+                               hasOverflows and overflowWidth > 0)
+        for item in multiweight_dict.values():
+            putils.adjustOverflows(item.values(), ax_format['xlim'],
+                                   int(plot_features.get('IncludeOverflows', '0')),
+                                   hasOverflows and underflowWidth > 0,
+                                   hasOverflows and overflowWidth > 0)
 
     ## Set maximum y-value from all hist datasets
     #max_ymax = max([h.points()[0].val(1) for h in hist_data.values()])
@@ -156,8 +204,6 @@ def mkPlottingScript1D(hist_data, hist_features, yaml_dicts, outdir, plot_name,
         ymax = 10*ymin if ymin else 1.0
 
     ax_format['ylim'] = (float(ymin), float(ymax))
-    ax_format['logx'] = int(plot_features.get('LogX', 0))
-    ax_format['logy'] = int(plot_features.get('LogY', 1))
 
     leg_pass = False
     leg_titles, leg_curves, leg_pos, ratio_panels = {}, {}, {}, {}
@@ -243,7 +289,7 @@ def mkPlottingScript1D(hist_data, hist_features, yaml_dicts, outdir, plot_name,
     elif plot_features.get('XMajorTickMarks') is not None:
         tmp = [ float(x) for x in plot_features.get('XMajorTickMarks').split() ]
         cmd.add("xMajorTickMarks = {}".format(tmp))
-    if plot_features.get('XMinorTickMarks') is not None:
+    if plot_features.get('XMinorTickMarks', ''):
         cmd.add(f"xMinorTickMarks = { int(plot_features.get('XMinorTickMarks')) }")
 
     # Check if user set custom y-tick marks
@@ -288,15 +334,17 @@ def mkPlottingScript1D(hist_data, hist_features, yaml_dicts, outdir, plot_name,
     if fc:
         cmd.add(f"fig.set_facecolor('{fc}')\n")
 
+    # default ticks rotations
+    xRot = plot_features.get('XMajorTicksAngle', 0)
+    yRot = plot_features.get('YMajorTicksAngle', 0)
+
     ## Set some ratio-panel formatting defaults
     deviation_modes = []
     for i, key in enumerate(ratio_panels.keys()):
         ratioName = 'ratio%d_ax' % i
         ## Ratio plot default y-ranges
-        xRot = plot_features.get('XMajorTicksAngle', 0)
-        yRot = plot_features.get('YMajorTicksAngle', 0)
-        ratioYMin = plot_features.get(key+'YMin', 0.5)
-        ratioYMax = plot_features.get(key+'YMax', 1.4999)  # avoid 1.5 tick mark
+        ratioYMin = float(plot_features.get(key+'YMin', 0.5))
+        ratioYMax = float(plot_features.get(key+'YMax', 1.4999))  # avoid 1.5 tick mark
         rpmode = plot_features.get(key+'Mode', 'mcdata')
         deviation_modes.append(rpmode == 'deviation')
         if deviation_modes[-1]:
@@ -305,7 +353,9 @@ def mkPlottingScript1D(hist_data, hist_features, yaml_dicts, outdir, plot_name,
                     f"                      r'$0\\,\\sigma$', r'$1\\,\\sigma$', r'$2\\,\\sigma$'])",
                     f"{ratioName}.set_ylim(-3.0, 3.0)")
         else:
-            cmd.add(f"{ratioName}.yaxis.set_major_locator(mpl.ticker.MultipleLocator(0.1))",
+            ratioTickDist = (ratioYMax-ratioYMin)/5.0 # default: five tick labels, 1 significant digit:
+            ratioTickDist = round(ratioTickDist, -int(math.floor(math.log10(abs(ratioTickDist)))))
+            cmd.add(f"{ratioName}.yaxis.set_major_locator(mpl.ticker.MultipleLocator({ratioTickDist}))",
                     f"{ratioName}.set_ylim({ratioYMin}, {ratioYMax})")
         ratioYLabel = plot_features.get(key+'YLabel', 'Ratio')
         cmd.add(f"{ratioName}.set_ylabel('{ratioYLabel}')")
@@ -317,9 +367,9 @@ def mkPlottingScript1D(hist_data, hist_features, yaml_dicts, outdir, plot_name,
                     f"{ratioName}.tick_params(axis='x', which='minor', bottom=False, top=False)")
         elif plot_features.get('XMajorTickMarks') is not None:
             cmd.add(f"{ratioName}.set_xticks(xMajorTickMarks, rotation={xRot}, minor=False)")
-            if plot_features.get('XMinorTickMarks') is None:
+            if not plot_features.get('XMinorTickMarks', ''):
                 cmd.add(f"{ratioName}.tick_params(axis='x', which='minor', bottom=False, top=False)")
-        if plot_features.get('XMinorTickMarks') is not None:
+        if plot_features.get('XMinorTickMarks', ''):
             n = int(plot_features.get('XMinorTickMarks'))
             if n == 0:
                 cmd.add(f"{ratioName}.xaxis.set_minor_locator(mpl.ticker.NullLocator())")
@@ -389,7 +439,7 @@ def mkPlottingScript1D(hist_data, hist_features, yaml_dicts, outdir, plot_name,
     ## Set text labels on axes
     cmd.newline()
     cmd.add("# set plot metadata as defined above")
-    xLabelSep = float(plot_features.get(f'XLabelSep', -0.15))
+    xLabelSep = float(plot_features.get(f'XLabelSep', -0.15 if len(ratio_panels.keys()) else -0.05 ))
     if len(ratio_panels.keys()):
         # only put the label for the last ratio panel
         ratioName = 'ratio%d_ax' % (len(ratio_panels.keys()) - 1)
@@ -408,8 +458,8 @@ def mkPlottingScript1D(hist_data, hist_features, yaml_dicts, outdir, plot_name,
             f"ax.set_yscale(ax_yScale)")
 
     ## Toggle x/y lims
-    if ax_format['xlim']: cmd.add("ax.set_xlim(xLims)")
-    if ax_format['ylim']: cmd.add("ax.set_ylim(yLims)")
+    cmd.add("ax.set_xlim(xLims)")
+    cmd.add("ax.set_ylim(yLims)")
 
     cmd.newline()
     cmd.add(f"# tick formatting",
@@ -422,9 +472,9 @@ def mkPlottingScript1D(hist_data, hist_features, yaml_dicts, outdir, plot_name,
                 f"ax.tick_params(axis='x', which='minor', bottom=False, top=False)")
     elif plot_features.get('XMajorTickMarks') is not None:
         cmd.add(f"ax.set_xticks(xMajorTickMarks, rotation={xRot}, minor=False)")
-        if plot_features.get('XMinorTickMarks') is None:
+        if not plot_features.get('XMinorTickMarks', ''):
             cmd.add(f"ax.tick_params(axis='x', which='minor', bottom=False, top=False)")
-    if plot_features.get('XMinorTickMarks') is not None:
+    if plot_features.get('XMinorTickMarks', ''):
         n = int(plot_features.get('XMinorTickMarks'))
         if n == 0:
             cmd.add(f"ax.xaxis.set_minor_locator(mpl.ticker.NullLocator())")
@@ -460,6 +510,51 @@ def mkPlottingScript1D(hist_data, hist_features, yaml_dicts, outdir, plot_name,
     if not xyticklabels or not yticklabels:
        cmd.add("ax.yaxis.set_tick_params(labelleft=False)")
 
+    # Annotate under-/overflow bins with labels
+    if underflowWidth > 0. or overflowWidth > 0.:
+        cmd.add(f"\n# Annotate under-/overflow bins")
+        ref_label = list(hist_data.keys())[0]
+        ylims = np.log10(ax_format['ylim']) if ax_format['logy'] else ax_format['ylim']
+        underflow_ypos = None
+        overflow_ypos = None
+        for h in hist_data.values():
+            y = h.yVals()[0]
+            if math.isfinite(y) and (y > 0. or not ax_format['logy']):
+                underflow_ypos = np.log10(y) if ax_format['logy'] else y
+            y = h.yVals()[-1]
+            if math.isfinite(y) and (y > 0. or not ax_format['logy']):
+                overflow_ypos = np.log10(y) if ax_format['logy'] else y
+            if underflow_ypos is not None and overflow_ypos is not None:
+                break
+        if underflowWidth > 0.:
+            if underflow_ypos is None:
+                underflow_ypos = 0.5*sum(ylims)
+            flip = int(underflow_ypos < 0.75*ylims[0] + 0.25*ylims[1])
+            sf = float(plot_features.get('UnderflowLabelOffset', 0.03))
+            underflow_ypos += sf*(ylims[1] - ylims[0]) * (-1.0) ** flip # shift above/below bin value
+            if ax_format['logy']:  underflow_ypos = 10 ** underflow_ypos
+            underflow_label = plot_features.get('UnderflowLabel', r'$-\infty\leq$')
+            underflow_color = plot_features.get('UnderflowLabelColor', 'black')
+            underflow_size = plot_features.get('UnderflowLabelSize', 8)
+            cmd.add(f"underflow_xpos = 0.5*sum(dataf['xedges']['{ref_label}'][:2])",
+                    f"underflow_ypos = {float(underflow_ypos):.2e}",
+                    f"ax.annotate(r'{underflow_label}', xy=(underflow_xpos, underflow_ypos),",
+                    f"            ha='center', fontsize={underflow_size}, color='{underflow_color}')")
+        if overflowWidth > 0.:
+            if overflow_ypos is None:
+                overflow_ypos = 0.5*sum(ylims)
+            flip = int(overflow_ypos < 0.75*ylims[0] + 0.25*ylims[1])
+            sf = float(plot_features.get('OverflowLabelOffset', 0.03))
+            overflow_ypos += sf*(ylims[1] - ylims[0]) * (-1.0) ** flip # shift above/below bin value
+            if ax_format['logy']:  overflow_ypos = 10 ** overflow_ypos
+            overflow_label = plot_features.get('OverflowLabel', r'$\leq\infty$')
+            overflow_color = plot_features.get('OverflowLabelColor', 'black')
+            overflow_size = plot_features.get('OverflowLabelSize', 8)
+            cmd.add(f"overflow_xpos = 0.5*sum(dataf['xedges']['{ref_label}'][-2:])",
+                    f"overflow_ypos = {float(overflow_ypos):.2e}",
+                    f"ax.annotate(r'{overflow_label}', xy=(overflow_xpos, overflow_ypos),",
+                    f"            ha='center', fontsize={overflow_size}, color='{overflow_color}')")
+
     for key in ratio_panels.keys():
         ratioName = 'ratio' + ('0' if 't' == key[-1] else key[-1])
         if ax_format[ratioName + '_logy']:
@@ -516,12 +611,23 @@ def mkPlottingScript2D(hist_data, hist_features, yaml_dicts, outdir,
                        plot_name, formats, has_latex = True):
     """ Return Python commands to make a Rivet plot from the input parameters."""
 
-      # TODO: lots of code duplication with 1D plotting: we should try to unify
+    # TODO: lots of code duplication with 1D plotting: we should try to unify
 
     cmd = yoda.util.StringCommand()
     setStyleFile(cmd, yaml_dicts, outdir)
     plot_features = yaml_dicts.get('plot features', {})
 
+    # check if user supplied regex-based label customisation
+    re_captures = []
+    for item in plot_features.get('PlotRegexes', []):
+        for match in RE_PATTERN_NAME.findall(item):
+            sub = re.findall(match, list(hist_data.values())[0].name())
+            if sub:  re_captures.append(sub[0])
+        for match in RE_PATTERN_PATH.findall(item):
+            sub = re.findall(match, list(hist_data.values())[0].path())
+            if sub:  re_captures.append(sub[0])
+    _apply_regexes(plot_features, re_captures, plot_features.get('PlotRegexSubs', {}))
+
     if not has_latex:
         cmd.newline()
         cmd.add(f"plt.rcParams['text.usetex'] = False",
@@ -543,44 +649,30 @@ def mkPlottingScript2D(hist_data, hist_features, yaml_dicts, outdir,
     cmd.add(f"ax_zScale = '{zscale}'")
 
     ax_format = {}  # Stores the items for formatting the axes in a dict
-    # Set plot limits
 
-    ## Set maximum Y value from all hist datasets
-    max_ymax = max([h.points()[0].val(1) for h in hist_data])
+    ## Set plot limits
+    xmin = plot_features.get('XMin', min([min(h.xMins()) for h in hist_data.values()]))
+    xmax = plot_features.get('XMax', max([max(h.xMaxs()) for h in hist_data.values()]))
+    ymin = plot_features.get('YMin', min([min(h.yMins()) for h in hist_data.values()]))
+    ymax = plot_features.get('YMax', max([max(h.yMaxs()) for h in hist_data.values()]))
+    zmin = plot_features.get('ZMin', min([min(h.zMins()) for h in hist_data.values()]))
+    zmax = plot_features.get('ZMax', max([max(h.zMaxs()) for h in hist_data.values()]))
+    ax_format['xlim'] = (float(xmin), float(xmax))
+    ax_format['ylim'] = (float(ymin), float(ymax))
+    ax_format['zlim'] = (float(zmin), float(zmax))
 
-    if plot_features.get('YMax') is not None:
-        ymax = plot_features.get('YMax')
-    elif plot_features.get('LogY', 1):
-        ## Round off the highest number in the histograms to the next power of 10
-        ymax = 10**(math.ceil(math.log10(max_ymax)))
-    else:
-        ymax = 1.1*max_ymax
-
-    ## Use minimum y-value from all hist datasets
-    min_ymin = min([h.points()[0].val(1) for h in hist_data]) # TO DO -- where does this come from??
-    if plot_features.get('YMin') is not None:
-        ymin = plot_features.get('YMin')
-    elif plot_features.get('LogY', 1):
-        ## Round off lowest number in the histograms to lower power of 10
-        # TODO: come up with a better solution to deal with min_ymin=0
-        ymin = 10**(math.floor(math.log10(min_ymin))) if min_ymin > 0 else 2e-7*ymax
-    elif plot_features.get('ShowZero', 1):  # default ShowZero is True
-        ymin = 0 if min_ymin > -1e-4 else 1.1*min_ymin
-    else:
-        ymin = (1.1*min_ymin if min_ymin < -1e-4 else 0 if min_ymin < 1e-4
-                else 0.9*min_ymin)
-
-    ax_format['ylim'] = (ymin, ymax)
     ax_format['logx'] = int(plot_features.get('LogX', 0))
     ax_format['logy'] = int(plot_features.get('LogY', 1))
     ax_format['logz'] = int(plot_features.get('LogZ', 0))
 
-    if ax_format.get('xlim'):  cmd.add(f"xLims = {ax_format['xlim']}")
-    if ax_format.get('ylim'):  cmd.add(f"yLims = {ax_format['ylim']}")
+    cmd.add(f"xLims = {ax_format['xlim']}")
+    cmd.add(f"yLims = {ax_format['ylim']}")
+    cmd.add(f"zLims = {ax_format['zlim']}")
 
     ## Labels for the legend
     labels = []
-    if plot_features.get('Legend', 1):
+    if plot_features.get('Legend', 0):
+        #for i, name in enumerate(hist_data.keys()):
         for i in range(len(hist_data)):
             lentry = str(hist_features[i].get('Title', 'Curve {}'.format(i+1)))
             lentry = lentry.replace('.yoda.gz','').replace('.yoda','')
@@ -637,7 +729,7 @@ def mkPlottingScript2D(hist_data, hist_features, yaml_dicts, outdir,
         cmd.add(datacmd)
         dataFile.write(data)
 
-    if plot_features.get('Legend', 1):
+    if plot_features.get('Legend', 0):
         if plot_features.get('LegendAlign') is None or plot_features.get('LegendAlign') == 'l':
             legend_pos = (float(plot_features.get('LegendXPos', 0.5)),
                           float(plot_features.get('LegendYPos', 0.97)))
@@ -658,13 +750,15 @@ def mkPlottingScript2D(hist_data, hist_features, yaml_dicts, outdir,
             f"ax.set_xscale(ax_xScale)",
             f"ax.set_yscale(ax_yScale)")
 
+    ## Toggle x/y lims
+    cmd.add("ax.set_xlim(xLims)")
+    cmd.add("ax.set_ylim(yLims)")
+
     cmd.newline()
     cmd.add(f"# tick formatting",
             f"plt.rcParams['xtick.top'] = {plot_features.get('XTwosidedTicks', True)}",
             f"plt.rcParams['ytick.right'] = {plot_features.get('YTwosidedTicks', True)}")
 
-    # https://stackoverflow.com/questions/8213522/when-to-use-cla-clf-or-close-for-clearing-a-plot-in-matplotlib
-    # plt.close(fig) instead of fig.clf()?
     for fmt in formats:
         figName = plot_name.split('/')[-1]+f'.{fmt.lower()}'
         cmd.add(f"plt.savefig(os.path.join(plot_outdir, '{figName}'), format='{fmt}')")
@@ -675,7 +769,7 @@ def mkPlottingScript2D(hist_data, hist_features, yaml_dicts, outdir,
 
 def process(yaml_file, plot_name, outdir = './', formats = ['pdf']):
     """\
-    Write an executable python script that only relies on YODA functionalities
+    Write an executable Python script that only relies on YODA functionalities
 
     Parameters
     ----------
@@ -686,7 +780,7 @@ def process(yaml_file, plot_name, outdir = './', formats = ['pdf']):
     outPyName = os.path.join(outdir, plot_name.strip('/')) + '.py'
 
     ## Contents to write out to .py script
-    mplScript = yoda.util.StringCommand("#! /usr/bin/env python\n")
+    mplScript = yoda.util.StringCommand("#! /usr/bin/env python3\n")
 
     yodaVersion = str(yoda.__version__)
     import datetime
@@ -702,7 +796,8 @@ def process(yaml_file, plot_name, outdir = './', formats = ['pdf']):
                   "import matplotlib.pyplot as plt",
                   "mpl.use('Agg') # comment out for interactive use",
                   "import os",
-                  "import numpy as np")
+                  "import numpy as np",
+                  "from numpy import nan, inf")
     mplScript.newline()
 
     rivetPlotsDir = os.path.join(os.getcwd(), outdir)
@@ -727,7 +822,8 @@ def process(yaml_file, plot_name, outdir = './', formats = ['pdf']):
         yaml_dicts = yaml_file
         yaml_dicts.setdefault('plot features', {})
         for k, hist_dict in yaml_dicts['histograms'].items():
-            hist_data[k] = putils.mkPlotFriendlyScatter(hist_dict['nominal'])
+            hasOverflows = int(yaml_dicts['plot features'].get('IncludeOverflows', '0'))
+            hist_data[k] = putils.mkPlotFriendlyScatter(hist_dict['nominal'], includeOverflows=hasOverflows)
             if 'Scale' in hist_dict:
                 hist_data[k].scale(hist_data[k].dim()-1, float(hist_dict['Scale']))
             if 'Rebin' in yaml_dicts['plot features']:
@@ -759,6 +855,7 @@ def process(yaml_file, plot_name, outdir = './', formats = ['pdf']):
                 except:
                     pass
 
+    if not os.path.exists(outdir):  os.makedirs(outdir)
     hist_features = list(yaml_dicts['histograms'].values())
     output_filename = os.path.join(outdir, plot_name.strip('/'))
     _preprocess_text(yaml_dicts)
@@ -768,7 +865,7 @@ def process(yaml_file, plot_name, outdir = './', formats = ['pdf']):
                                           plot_name, hist_data_multiweights, formats, has_latex) )
 
     elif all(h.dim() == 3 for h in hist_data.values()):
-        mplScript.add( mkPlottingScript2D(hist_data.values(), hist_features, yaml_dicts,
+        mplScript.add( mkPlottingScript2D(hist_data, hist_features, yaml_dicts,
                                           outdir, plot_name, formats, has_latex) )
     else:
         print('Error with Class types:', [h.type() for h in hist_data.values()])
@@ -799,9 +896,7 @@ def _parse_yoda_hist(yaml_dicts):
 
 def _preprocess_text(yaml_dicts):
     """Preprocess text to convert convenient HEP units and other symbols to mathtext."""
-    if 'plot features' not in yaml_dicts:
-        yaml_dicts['plot features'] = {}
-    plot_features = yaml_dicts['plot features']
+    plot_features = yaml_dicts.get('plot features', {})
     for label in plot_features:
         if any(label.endswith(sub) for sub in ('Title', 'Label', 'CustomMajorTicks')):
             plot_features[label] = preprocess(plot_features[label])
@@ -811,6 +906,13 @@ def _preprocess_text(yaml_dicts):
             hist_setting['Title'] = preprocess(hist_setting['Title'])
 
 
+def _apply_regexes(features, regex_captures, regex_subs):
+    """Apply regular expressions to plot labels, titles and such using the applyRegex() helper method."""
+    for label in features:
+        if any(label.endswith(sub) for sub in ('Title', 'Label', 'CustomMajorTicks')):
+            features[label] = applyRegexes(features[label], regex_captures, regex_subs)
+
+
 class AnyObject(object):
     """Necessary for custom legend handler."""
     pass
