@@ -1,4 +1,4 @@
-#include "YODA/Histo1D.h"
+#include "YODA/Histo.h"
 #include "YODA/Utils/Formatting.h"
 #include <cmath>
 #include <iostream>
@@ -19,18 +19,16 @@ int main() {
   MSG("Mean value = " << h.xMean() << " +- " << h.xStdErr());
   MSG("Total area = " << h.integral());
 
-  auto compareHeight = [](const HistoBin1D& a, const HistoBin1D& b) { return a.height() < b.height(); };
-  const HistoBin1D& highestBin = *( max_element(h.bins().begin(), h.bins().end(), compareHeight) );
-  const double maxHeight = highestBin.height();
-
+  const double maxHeight = h.maxDensity();
   for (int i = 0; i < 4; ++i) {
-    if (i > 0) h.rebin(2);
+    if (i > 0) h.rebin<0>(2);
     MSG("Histo (rebinning #" << i << ", num bins = " << h.numBins() << ")");
-    for (const HistoBin1D& b : h.bins()) {
-      const int numElements = static_cast<int>(round(20 * b.height()/maxHeight));
-      MSG(string().insert(0, numElements, '=') << "  " << RED(b.height()));
+    for (const auto& b : h.bins()) {
+      const int numElements = static_cast<int>(round(20 * b.sumW()/maxHeight));
+      MSG(string().insert(0, numElements, '=') << "  " << RED(b.sumW()));
     }
   }
 
   return EXIT_SUCCESS;
 }
+
