@@ -10,7 +10,7 @@ int main() {
 
   vector<AnalysisObject*> aos1 = YODA::read("testwriter2.yoda");
 
-  #ifdef WITH_ZLIB
+  #ifdef HAVE_LIBZ
   vector<AnalysisObject*> aos2 = YODA::read("testwriter2.yoda.gz");
   #endif
 
